@@ -6,11 +6,19 @@ sys.path.append(os.path.abspath("exts"))
 
 master_doc = "index"
 
+man_pages = [
+    ("man/drgn", "drgn", "programmable debugger", "", "1"),
+]
+
+option_emphasise_placeholders = True
+
 extensions = [
+    "details",
     "drgndoc.ext",
     "linuxsrc",
     "setuptools_config",
     "sphinx.ext.extlinks",
+    "sphinx.ext.graphviz",
     "sphinx.ext.intersphinx",
 ]
 
