@@ -48,12 +48,19 @@ from typing import Union
 
 from _drgn import (
     NULL,
+    AbsenceReason,
     Architecture,
+    DebugInfoOptions,
+    ExtraModule,
     FaultError,
     FindObjectFlags,
     IntegerLike,
+    KmodSearchMethod,
     Language,
+    MainModule,
     MissingDebugInfoError,
+    Module,
+    ModuleFileStatus,
     NoDefaultProgramError,
     Object,
     ObjectAbsentError,
@@ -66,8 +73,11 @@ from _drgn import (
     ProgramFlags,
     Qualifiers,
     Register,
+    RelocatableModule,
+    SharedLibraryModule,
     StackFrame,
     StackTrace,
+    SupplementaryFileKind,
     Symbol,
     SymbolBinding,
     SymbolIndex,
@@ -80,6 +90,8 @@ from _drgn import (
     TypeMember,
     TypeParameter,
     TypeTemplateParameter,
+    VdsoModule,
+    WantedSupplementaryFile,
     alignof,
     cast,
     container_of,
@@ -100,17 +112,26 @@ from _drgn import (
 # isort: split
 from _drgn import (  # noqa: F401
     _elfutils_version as _elfutils_version,
+    _enable_dlopen_debuginfod as _enable_dlopen_debuginfod,
+    _have_debuginfod as _have_debuginfod,
     _with_libkdumpfile as _with_libkdumpfile,
 )
 from drgn.internal.version import __version__ as __version__  # noqa: F401
 
 __all__ = (
+    "AbsenceReason",
     "Architecture",
+    "DebugInfoOptions",
+    "ExtraModule",
     "FaultError",
     "FindObjectFlags",
     "IntegerLike",
+    "KmodSearchMethod",
     "Language",
+    "MainModule",
     "MissingDebugInfoError",
+    "Module",
+    "ModuleFileStatus",
     "NULL",
     "NoDefaultProgramError",
     "Object",
@@ -124,8 +145,11 @@ __all__ = (
     "ProgramFlags",
     "Qualifiers",
     "Register",
+    "RelocatableModule",
+    "SharedLibraryModule",
     "StackFrame",
     "StackTrace",
+    "SupplementaryFileKind",
     "Symbol",
     "SymbolBinding",
     "SymbolIndex",
@@ -138,6 +162,8 @@ __all__ = (
     "TypeMember",
     "TypeParameter",
     "TypeTemplateParameter",
+    "VdsoModule",
+    "WantedSupplementaryFile",
     "alignof",
     "cast",
     "container_of",
