@@ -175,29 +175,33 @@ drgn_orc_to_cfi_x86_64(const struct drgn_orc_entry *orc,
 					 drgn_orc_type(orc));
 	}
 
-	switch (drgn_orc_bp_reg(orc)) {
-	case DRGN_ORC_REG_UNDEFINED:
-		rule.kind = DRGN_CFI_RULE_REGISTER_PLUS_OFFSET;
-		rule.regno = DRGN_REGISTER_NUMBER(rbp);
-		rule.offset = 0;
-		break;
-	case DRGN_ORC_REG_PREV_SP:
-		rule.kind = DRGN_CFI_RULE_AT_CFA_PLUS_OFFSET;
-		rule.offset = orc->bp_offset;
-		break;
-	case DRGN_ORC_REG_BP:
-		rule.kind = DRGN_CFI_RULE_AT_REGISTER_PLUS_OFFSET;
-		rule.regno = DRGN_REGISTER_NUMBER(rbp);
-		rule.offset = orc->bp_offset;
-		break;
-	default:
-		return drgn_error_format(DRGN_ERROR_OTHER,
-					 "unknown ORC BP base register %d",
-					 drgn_orc_bp_reg(orc));
+	// For ORC_TYPE_REGS, rbp is already set.
+	if (drgn_orc_type(orc) != DRGN_ORC_TYPE_REGS) {
+		switch (drgn_orc_bp_reg(orc)) {
+		case DRGN_ORC_REG_UNDEFINED:
+			rule.kind = DRGN_CFI_RULE_REGISTER_PLUS_OFFSET;
+			rule.regno = DRGN_REGISTER_NUMBER(rbp);
+			rule.offset = 0;
+			break;
+		case DRGN_ORC_REG_PREV_SP:
+			rule.kind = DRGN_CFI_RULE_AT_CFA_PLUS_OFFSET;
+			rule.offset = orc->bp_offset;
+			break;
+		case DRGN_ORC_REG_BP:
+			rule.kind = DRGN_CFI_RULE_AT_REGISTER_PLUS_OFFSET;
+			rule.regno = DRGN_REGISTER_NUMBER(rbp);
+			rule.offset = orc->bp_offset;
+			break;
+		default:
+			return drgn_error_format(DRGN_ERROR_OTHER,
+						 "unknown ORC BP base register %d",
+						 drgn_orc_bp_reg(orc));
+		}
+		if (!drgn_cfi_row_set_register(row_ret, DRGN_REGISTER_NUMBER(rbp),
+					       &rule))
+			return &drgn_enomem;
 	}
-	if (!drgn_cfi_row_set_register(row_ret, DRGN_REGISTER_NUMBER(rbp),
-				       &rule))
-		return &drgn_enomem;
+
 	*interrupted_ret = drgn_orc_signal(orc);
 	*ret_addr_regno_ret = DRGN_REGISTER_NUMBER(rip);
 	return NULL;
@@ -228,11 +232,36 @@ get_registers_from_frame_pointer(struct drgn_program *prog,
 	return NULL;
 }
 
-// Unwind from a call instruction, assuming that nothing else has been changed
-// since.
-static struct drgn_error *unwind_call(struct drgn_program *prog,
-				      struct drgn_register_state *regs,
-				      struct drgn_register_state **ret)
+
+static struct drgn_error *
+fallback_unwind_x86_64(struct drgn_program *prog,
+		       struct drgn_register_state *regs,
+		       struct drgn_register_state **ret)
+{
+	struct drgn_error *err;
+
+	struct optional_uint64 rbp =
+		drgn_register_state_get_u64(prog, regs, rbp);
+	if (!rbp.has_value)
+		return &drgn_stop;
+
+	err = get_registers_from_frame_pointer(prog, rbp.value, ret);
+	if (err) {
+		if (err->code == DRGN_ERROR_FAULT) {
+			drgn_error_destroy(err);
+			err = &drgn_stop;
+		}
+		return err;
+	}
+	drgn_register_state_set_cfa(prog, regs, rbp.value + 16);
+	return NULL;
+}
+
+// Unwind a single call instruction.
+static struct drgn_error *
+bad_call_unwind_x86_64(struct drgn_program *prog,
+		       struct drgn_register_state *regs,
+		       struct drgn_register_state **ret)
 {
 	struct drgn_error *err;
 
@@ -266,37 +295,6 @@ static struct drgn_error *unwind_call(struct drgn_program *prog,
 	return NULL;
 }
 
-static struct drgn_error *
-fallback_unwind_x86_64(struct drgn_program *prog,
-		       struct drgn_register_state *regs,
-		       struct drgn_register_state **ret)
-{
-	struct drgn_error *err;
-
-	// If the program counter is 0, it's likely that a NULL function pointer
-	// was called. Assume that the only thing we need to unwind is a single
-	// call instruction.
-	struct optional_uint64 pc = drgn_register_state_get_pc(regs);
-	if (pc.has_value && pc.value == 0)
-		return unwind_call(prog, regs, ret);
-
-	struct optional_uint64 rbp =
-		drgn_register_state_get_u64(prog, regs, rbp);
-	if (!rbp.has_value)
-		return &drgn_stop;
-
-	err = get_registers_from_frame_pointer(prog, rbp.value, ret);
-	if (err) {
-		if (err->code == DRGN_ERROR_FAULT) {
-			drgn_error_destroy(err);
-			err = &drgn_stop;
-		}
-		return err;
-	}
-	drgn_register_state_set_cfa(prog, regs, rbp.value + 16);
-	return NULL;
-}
-
 // elf_gregset_t (in NT_PRSTATUS) and struct user_regs_struct have the same
 // layout. struct pt_regs is a prefix of that layout which elides several
 // segment registers. full_regset tells us which one we were given; true is
@@ -663,6 +661,7 @@ const struct drgn_architecture_info arch_info_x86_64 = {
 	DRGN_ARCHITECTURE_REGISTERS,
 	.default_dwarf_cfi_row = &default_dwarf_cfi_row_x86_64,
 	.fallback_unwind = fallback_unwind_x86_64,
+	.bad_call_unwind = bad_call_unwind_x86_64,
 	.pt_regs_get_initial_registers = pt_regs_get_initial_registers_x86_64,
 	.prstatus_get_initial_registers = prstatus_get_initial_registers_x86_64,
 	.linux_kernel_get_initial_registers =
