@@ -73,51 +73,74 @@ void clear_drgn_in_python(void)
 	drgn_in_python = false;
 }
 
+static struct drgn_error *drgn_fault_error_from_python(PyObject *exc_value)
+{
+	_cleanup_pydecref_ PyObject *py_message =
+		PyObject_GetAttrString(exc_value, "message");
+	const char *message = py_message ? PyUnicode_AsUTF8(py_message) : NULL;
+	if (!message)
+		return NULL;
+
+	_cleanup_pydecref_ PyObject *py_address =
+		PyObject_GetAttrString(exc_value, "address");
+	uint64_t address = py_address ? PyLong_AsUint64(py_address) : (uint64_t)-1;
+	if (address == (uint64_t)-1 && PyErr_Occurred())
+		return NULL;
+
+	return drgn_error_create_fault(message, address);
+}
+
 struct drgn_error *drgn_error_from_python(void)
 {
-	PyObject *exc_type, *exc_value, *exc_traceback, *exc_message;
-	const char *type, *message;
-	struct drgn_error *err;
-
+	_cleanup_pydecref_ PyObject *exc_type, *exc_value, *exc_traceback;
 	PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
 	if (!exc_type)
 		return NULL;
 
+	// Python FaultErrors should be translated back to drgn errors because
+	// they are frequently handled in libdrgn. They should be translated no
+	// matter how deeply nested we are, so we do this before checking
+	// drgn_in_python.
+	if ((PyTypeObject *)exc_type == &FaultError_type && exc_value) {
+		struct drgn_error *err = drgn_fault_error_from_python(exc_value);
+		if (err)
+			return err;
+		// A NULL return means that we encountered a Python error while
+		// trying to convert it. Clear the Python error and fall back to
+		// the standard code path.
+		PyErr_Clear();
+	}
+
 	if (drgn_in_python) {
 		PyErr_Restore(exc_type, exc_value, exc_traceback);
+		exc_type = exc_value = exc_traceback = NULL;
 		return &drgn_error_python;
 	}
 
-	type = ((PyTypeObject *)exc_type)->tp_name;
+	const char *type = ((PyTypeObject *)exc_type)->tp_name;
+	_cleanup_pydecref_ PyObject *exc_message = NULL;
+	const char *message;
 	if (exc_value) {
 		exc_message = PyObject_Str(exc_value);
 		message = exc_message ? PyUnicode_AsUTF8(exc_message) : NULL;
 		if (!message) {
-			err = drgn_error_format(DRGN_ERROR_OTHER,
-						"%s: <exception str() failed>", type);
-			goto out;
+			PyErr_Clear();
+			return drgn_error_format(DRGN_ERROR_OTHER,
+						 "%s: <exception str() failed>", type);
 		}
 	} else {
-		exc_message = NULL;
 		message = "";
 	}
 
 	if (message[0]) {
-		err = drgn_error_format(DRGN_ERROR_OTHER, "%s: %s", type,
-					message);
+		return drgn_error_format(DRGN_ERROR_OTHER, "%s: %s", type,
+					 message);
 	} else {
-		err = drgn_error_create(DRGN_ERROR_OTHER, type);
+		return drgn_error_create(DRGN_ERROR_OTHER, type);
 	}
-
-out:
-	Py_XDECREF(exc_message);
-	Py_XDECREF(exc_traceback);
-	Py_XDECREF(exc_value);
-	Py_DECREF(exc_type);
-	return err;
 }
 
-DRGNPY_PUBLIC void *set_drgn_error(struct drgn_error *err)
+void *set_drgn_error(struct drgn_error *err)
 {
 	if (err == &drgn_error_python)
 		return NULL;
