@@ -3,6 +3,7 @@
 
 import contextlib
 import functools
+import logging
 import os
 import sys
 from typing import Any, Mapping, NamedTuple, Optional
@@ -10,12 +11,14 @@ import unittest
 from unittest.mock import Mock
 
 from drgn import (
+    AbsenceReason,
     Architecture,
     FindObjectFlags,
     Language,
     Object,
     Platform,
     PlatformFlags,
+    PrimitiveType,
     Program,
     Type,
     TypeEnumerator,
@@ -124,7 +127,9 @@ def assertReprPrettyEqualsStr(obj):
 
 _IDENTICAL_EQ_TYPES = (
     type(None),
+    AbsenceReason,
     Language,
+    PrimitiveType,
     Program,
     TypeEnumerator,
     TypeKind,
@@ -196,6 +201,7 @@ def identical(a, b):
                     "prog_",
                     "type_",
                     "address_",
+                    "absence_reason_",
                     "bit_offset_",
                     "bit_field_size_",
                 ),
@@ -455,3 +461,14 @@ def modifyenv(vars: Mapping[str, Optional[str]]):
                 del os.environ[key]
             else:
                 os.environ[key] = old_value
+
+
+@contextlib.contextmanager
+def drgn_log_level(level: int):
+    logger = logging.getLogger("drgn")
+    old_level = logger.getEffectiveLevel()
+    logger.setLevel(level)
+    try:
+        yield
+    finally:
+        logger.setLevel(old_level)
