@@ -5,9 +5,9 @@ import gzip
 import re
 import unittest
 
+from _drgn_util.platform import NORMALIZED_MACHINE_NAME
 from drgn.helpers.linux.kconfig import get_kconfig
 from tests.linux_kernel import LinuxKernelTestCase
-from util import NORMALIZED_MACHINE_NAME
 
 
 class TestKconfig(LinuxKernelTestCase):
