@@ -17,10 +17,16 @@ from drgn.helpers.linux.net import (
     netdev_for_each_tx_queue,
     netdev_get_by_index,
     netdev_get_by_name,
+    netdev_priv,
     sk_fullsock,
+    skb_shinfo,
 )
 from drgn.helpers.linux.pid import find_task
-from tests.linux_kernel import LinuxKernelTestCase, create_socket
+from tests.linux_kernel import (
+    LinuxKernelTestCase,
+    create_socket,
+    skip_unless_have_test_kmod,
+)
 
 
 class TestNet(LinuxKernelTestCase):
@@ -48,6 +54,19 @@ class TestNet(LinuxKernelTestCase):
             netdev = netdev_get_by_name(self.net, name)
             self.assertEqual(netdev.ifindex, index)
 
+    @skip_unless_have_test_kmod
+    def test_netdev_get_by_name_init_net(self):
+        self.assertEqual(
+            netdev_get_by_name(self.prog, "lo"), self.prog["drgn_test_netdev"]
+        )
+
+    @skip_unless_have_test_kmod
+    def test_netdev_priv(self):
+        self.assertEqual(
+            netdev_priv(self.prog["drgn_test_netdev"]),
+            self.prog["drgn_test_netdev_priv"],
+        )
+
     def test_for_each_net(self):
         self.assertIn(self.prog["init_net"].address_of_(), for_each_net(self.prog))
 
@@ -96,3 +115,9 @@ class TestNet(LinuxKernelTestCase):
             sock = SOCKET_I(fget(self.task, skt.fileno()).f_inode)
             inode = SOCK_INODE(sock)
             self.assertEqual(inode.i_mode & _S_IFMT, _S_IFSOCK)
+
+    @skip_unless_have_test_kmod
+    def test_skb_shinfo(self):
+        self.assertEqual(
+            skb_shinfo(self.prog["drgn_test_skb"]), self.prog["drgn_test_skb_shinfo"]
+        )
