@@ -6,12 +6,13 @@ import os
 from pathlib import Path
 import re
 import shlex
+import shutil
 import signal
 import socket
 import subprocess
 import sys
 import tempfile
-from typing import Optional
+from typing import Any, Optional, Sequence
 
 from util import nproc, out_of_date
 from vmtest.config import HOST_ARCHITECTURE, Kernel, local_kernel
@@ -90,9 +91,7 @@ umount -n -l /mnt
 # Load kernel modules.
 mkdir -p "/lib/modules/$RELEASE"
 mount --bind {kernel_dir} "/lib/modules/$RELEASE"
-for module in configs rng_core virtio_rng; do
-	modprobe "$module"
-done
+modprobe -a rng_core virtio_rng
 
 # Create static device nodes.
 grep -v '^#' "/lib/modules/$RELEASE/modules.devname" |
@@ -142,6 +141,7 @@ fi
 cd {cwd}
 {test_kmod}
 set +e
+{stty}
 setsid -c sh -c {command}
 rc=$?
 set -e
@@ -211,6 +211,7 @@ def run_in_vm(
     root_dir: Optional[Path],
     build_dir: Path,
     *,
+    extra_qemu_options: Sequence[str] = (),
     test_kmod: TestKmodMode = TestKmodMode.NONE,
 ) -> int:
     if root_dir is None:
@@ -219,13 +220,8 @@ def run_in_vm(
         else:
             root_dir = build_dir / kernel.arch.name / "rootfs"
 
-    if test_kmod == TestKmodMode.NONE:
-        test_kmod_command = ""
-    else:
+    if test_kmod != TestKmodMode.NONE:
         kmod = build_kmod(build_dir, kernel)
-        test_kmod_command = f"export DRGN_TEST_KMOD={shlex.quote(str(kmod))}"
-        if test_kmod == TestKmodMode.INSERT:
-            test_kmod_command += '\ninsmod "$DRGN_TEST_KMOD"'
 
     qemu_exe = "qemu-system-" + kernel.arch.name
     match = re.search(
@@ -291,6 +287,21 @@ def run_in_vm(
             init = f'/bin/sh -- -c "/bin/mount -t tmpfs tmpfs /tmp && /bin/mkdir /tmp/host && /bin/mount -t 9p -o {_9pfs_mount_options},ro host /tmp/host && . /tmp/host{init_path.resolve()}"'
             host_dir_prefix = "/host"
 
+        if test_kmod == TestKmodMode.NONE:
+            test_kmod_command = ""
+        else:
+            test_kmod_command = f"export DRGN_TEST_KMOD={shlex.quote(host_dir_prefix + str(kmod.resolve()))}"
+            if test_kmod == TestKmodMode.INSERT:
+                test_kmod_command += '\ninsmod "$DRGN_TEST_KMOD"'
+
+        terminal_size = shutil.get_terminal_size((0, 0))
+        if terminal_size.columns or terminal_size.lines:
+            stty_command = (
+                f"stty cols {terminal_size.columns} rows {terminal_size.lines}"
+            )
+        else:
+            stty_command = ""
+
         with init_path.open("w") as init_file:
             init_file.write(
                 _INIT_TEMPLATE.format(
@@ -301,6 +312,7 @@ def run_in_vm(
                     command=shlex.quote(command),
                     kdump_needs_nosmp="" if kvm_args else "export KDUMP_NEEDS_NOSMP=1",
                     test_kmod=test_kmod_command,
+                    stty=stty_command,
                 )
             )
         init_path.chmod(0o755)
@@ -345,6 +357,8 @@ def run_in_vm(
                 "-kernel", str(kernel.path / "vmlinuz"),
                 "-append",
                 f"rootfstype=9p rootflags={_9pfs_mount_options} ro console={kernel.arch.qemu_console},115200 panic=-1 crashkernel=256M init={init}",
+
+                *extra_qemu_options,
                 # fmt: on
             ],
             env=env,
@@ -389,6 +403,16 @@ if __name__ == "__main__":
         format="%(asctime)s:%(levelname)s:%(name)s:%(message)s", level=logging.INFO
     )
 
+    class _StringSplitExtendAction(argparse.Action):
+        def __call__(
+            self, parser: Any, namespace: Any, values: Any, option_string: Any = None
+        ) -> None:
+            items = getattr(namespace, self.dest, None)
+            if items is None:
+                setattr(namespace, self.dest, values.split())
+            else:
+                items.extend(values.split())
+
     parser = argparse.ArgumentParser(
         description="run vmtest virtual machine",
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
@@ -426,6 +450,21 @@ if __name__ == "__main__":
         type=Path,
         help="directory to use as root directory in VM (default: / for the host architecture, $directory/$arch/rootfs otherwise)",
     )
+    parser.add_argument(
+        "--qemu-options",
+        metavar="OPTIONS",
+        action=_StringSplitExtendAction,
+        default=argparse.SUPPRESS,
+        help="additional options to pass to QEMU, split on spaces. May be given multiple times",
+    )
+    parser.add_argument(
+        "-Xqemu",
+        metavar="OPTION",
+        action="append",
+        dest="qemu_options",
+        default=argparse.SUPPRESS,
+        help="additional option to pass to QEMU (not split on spaces). May be given multiple times",
+    )
     parser.add_argument(
         "--build-test-kmod",
         dest="test_kmod",
@@ -453,14 +492,17 @@ if __name__ == "__main__":
     if not hasattr(args, "kernel"):
         assert HOST_ARCHITECTURE is not None
         args.kernel = DownloadKernel(HOST_ARCHITECTURE, "*")
+    if not hasattr(args, "root_directory"):
+        args.root_directory = None
+    if not hasattr(args, "qemu_options"):
+        args.qemu_options = []
+    if not hasattr(args, "test_kmod"):
+        args.test_kmod = TestKmodMode.NONE
+
     if args.kernel.pattern.startswith(".") or args.kernel.pattern.startswith("/"):
         kernel = local_kernel(args.kernel.arch, Path(args.kernel.pattern))
     else:
         kernel = next(download(args.directory, [args.kernel]))  # type: ignore[assignment]
-    if not hasattr(args, "root_directory"):
-        args.root_directory = None
-    if not hasattr(args, "test_kmod"):
-        args.test_kmod = TestKmodMode.NONE
 
     try:
         command = (
@@ -474,6 +516,7 @@ if __name__ == "__main__":
                 kernel,
                 args.root_directory,
                 args.directory,
+                extra_qemu_options=args.qemu_options,
                 test_kmod=args.test_kmod,
             )
         )
